ESI_DATASOURCE = 'tranquility'
ESI_ENDPOINT = 'https://esi.evetech.net/latest/swagger.json'
ESI_TIMEOUT = 10
ESI_SWAGGER_CACHE_TIMER = 86400
ESI_AUTHORIZE_ENDPOINT = 'https://login.eveonline.com/oauth/authorize'
ESI_ACCESS_TOKEN_ENDPOINT = 'https://login.eveonline.com/oauth/token'
ESI_TOKEN_VERIFY_ENDPOINT = 'https://login.eveonline.com/oauth/verify'
